"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.desc = exports.command = void 0;
const path_1 = require("path");
const add_1 = __importDefault(require("../actions/git/add"));
const commit_1 = __importDefault(require("../actions/git/commit"));
const init_1 = __importDefault(require("../actions/git/init"));
const file_1 = __importDefault(require("../actions/node/file"));
const folder_1 = __importDefault(require("../actions/node/folder"));
const template_1 = __importDefault(require("../actions/node/template"));
const init_2 = __importDefault(require("../actions/npm/init"));
const install_1 = __importDefault(require("../actions/npm/install"));
const spinner_1 = require("../lib/cli/spinner");
const yargs_1 = require("../lib/cli/yargs");
exports.command = 'init';
exports.desc = 'Initializes the project';
const options = { type: 'boolean', default: false, group: `${exports.command}-Options` };
exports.builder = yargs_1.builderDefault(exports.command, (yargs) => yargs.options({
    package: { ...options, alias: 'p', desc: 'Project is created as a package' },
    cli: { ...options, alias: 'c', implies: 'package', desc: 'Extends the package with CLI features' },
    'no-ts': { ...options, alias: 't', desc: 'Typescript will be not installed' },
    'no-jest': { ...options, alias: 'j', desc: 'Jest will be not installed' },
    'no-commit': { ...options, desc: 'No initial commit is executed at the end' },
    github: { type: 'string', implies: 'package', desc: 'Github username', group: exports.command },
}));
exports.handler = async (argv) => {
    var _a;
    const githubUsername = (_a = argv.github) === null || _a === void 0 ? void 0 : _a.trim();
    if (githubUsername !== undefined && githubUsername === '') {
        throw new Error('The github name is empty');
    }
    const files = [];
    const folders = ['.vscode', 'src', 'src/lib', 'tests'];
    const templates = [
        ['vscode/settings.json', '.vscode/settings.json'],
        ['editorconfig', '.editorconfig'],
        ['huskyrc', '.huskyrc'],
        ['eslintignore', '.eslintignore'],
        ['gitignore', '.gitignore'],
        ['npmignore', '.npmignore'],
        ['prettierignore', '.prettierignore'],
    ];
    const libraryProduction = [];
    const libraryDevelopment = ['husky', 'carna'];
    const packageBin = {};
    const packageInit = {
        private: true,
        version: '0.1.0',
        engines: { node: '>= 12.9' },
    };
    const packageUpdate = {};
    const packageScripts = {
        lint: 'npx carna lint',
    };
    // Typescript
    if (!argv['no-typescript']) {
        files.push('src/index.ts', 'src/lib/types.ts');
        templates.push(['tsconfig.json'], ['tsconfig.build.json']);
        libraryDevelopment.push('@types/node', 'typescript');
        packageUpdate.main = 'build/index.js';
        packageUpdate.types = 'build/index.d.ts';
        packageScripts.build = 'tsc --project ./tsconfig.build.json';
        packageScripts.prebuild = 'rm -rf ./build';
    }
    else {
        files.push('src/index.js');
        packageUpdate.main = 'src/index.js';
    }
    // Github
    if (githubUsername) {
        folders.push('.dependabot', '.github/workflows');
        templates.push(['dependabot/config.yml', '.dependabot/config.yml']);
        templates.push(['github/CODEOWNERS', '.github/CODEOWNERS']);
        templates.push([`github/workflows/qa-${argv['no-typescript'] ? 'js' : 'ts'}.yml`, '.github/workflows/qa.yml']);
    }
    // Library
    if (argv.package) {
        if (githubUsername) {
            packageInit.repository = { type: 'git', url: `https://github.com/${githubUsername}/<repo>` };
            packageInit.bugs = { url: `https://github.com/${githubUsername}/<repo>/issues/new` };
            templates.push(['github/workflows/release.yml', '.github/workflows/release.yml']);
        }
        templates.push(['releaserc.json', '.releaserc.json']);
        packageInit.publishConfig = { access: 'public' };
        packageInit.author = 'name <email>';
        // libraryDevelopment.push('@kettil/semantic-release-config');
    }
    else {
        delete packageUpdate.main;
        delete packageUpdate.type;
    }
    // CLI
    if (argv.cli) {
        folders.push('src/bin');
        templates.push([argv['no-typescript'] ? 'src/bin/index.js' : 'src/bin/index.ts']);
        packageBin[path_1.basename(argv.cwd)] = argv['no-typescript'] ? 'src/bin/index.js' : 'build/bin/index.js';
    }
    // Actions
    // NPM
    await spinner_1.spinnerAction(init_2.default(argv, { packageBin, packageInit, packageUpdate, packageScripts }), 'Create the package.json');
    // GIT
    await spinner_1.spinnerAction(init_1.default(argv, {}), 'Create the git repository');
    // FOLDER
    await spinner_1.spinnerAction(Promise.all(folders.map((folder) => folder_1.default(argv, { folder }))), 'Create the project folders');
    // FILES
    await spinner_1.spinnerAction(Promise.all(files.map((file) => file_1.default(argv, { file }))), 'Create the project files');
    // TEMPLATES
    /* eslint-disable-next-line @typescript-eslint/naming-convention */
    const variables = { GITHUB_USERNAME: githubUsername };
    await spinner_1.spinnerAction(Promise.all(templates.map(([source, target]) => template_1.default(argv, { source, target, variables }))), 'Create template files');
    // NPM INSTALL PROD
    await spinner_1.spinnerAction(install_1.default(argv, { packages: libraryProduction, mode: 'prod' }), 'Install prod dependencies');
    // NPM INSTALL DEV
    await spinner_1.spinnerAction(install_1.default(argv, { packages: libraryDevelopment, mode: 'dev' }), 'Install dev dependencies');
    if (!argv['no-commit']) {
        // GIT ADD
        await spinner_1.spinnerAction(add_1.default(argv, { files: ['.'] }), 'Add files to repository');
        // GIT COMMIT
        await spinner_1.spinnerAction(commit_1.default(argv, { msg: 'feat: 🐣' }), 'Create the initial commit');
    }
};
